// ============================================================================
// ==                                                                        ==
// == Amdis - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file ArhReader.h */

#ifndef AMDIS_ARH_READER_H
#define AMDIS_ARH_READER_H

#include "AMDiS_fwd.h"

namespace AMDiS {

  using namespace std;

  class ArhReader
  {
  public:
    static void read(string filename, 
		     Mesh *mesh,
		     DOFVector<double>* vec0 = NULL,
		     DOFVector<double>* vec1 = NULL,
		     DOFVector<double>* vec2 = NULL,
#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
		     bool writeParallel = true,
#else
		     bool writeParallel = false,
#endif
		     int nProcs = -1);

    static void read(string filename, 
		     Mesh *mesh,
		     vector<DOFVector<double>*> vecs,
#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
		     bool writeParallel = true,
#else
		     bool writeParallel = false,
#endif
		     int nProcs = -1);

    static void readMeta(string filename,
			 Mesh *mesh,
			 DOFVector<double>* vec0 = NULL,
			 DOFVector<double>* vec1 = NULL,
			 DOFVector<double>* vec2 = NULL);

    static void readMeta(string filename,
			 Mesh *mesh,
			 vector<DOFVector<double>*> vecs);

    static int readMetaData(string filename,
			    map<int, int> &elInRank,
			    map<int, int> &elCodeSize,
			    string &arhPrefix);

    static int readMetaData(string filename,
			    map<int, int> &elInRank,
			    map<int, int> &elCodeSize)
    {
      string tmp;
      return readMetaData(filename, elInRank, elCodeSize, tmp);
    }

    /// Returns just the number of subdomains a meta ARH file is defined for.
    static int readMetaData(string filename);

    static void readFromMemoryBlock(vector<char> &data, Mesh *mesh,
				    DOFVector<double>* vec0 = NULL,
				    DOFVector<double>* vec1 = NULL,
				    DOFVector<double>* vec2 = NULL,
#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
				    bool writeParallel = true,
#else
				    bool writeParallel = false,
#endif
				    int nProcs = -1);
    
    static void readFromMemoryBlock(vector<char> &data, Mesh *mesh,
				    vector<DOFVector<double>*> vecs,
#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
				    bool writeParallel = true,
#else
				    bool writeParallel = false,
#endif
				    int nProcs = -1);
    
    static int getNumValueVectors(string filename);

  private:

    static void readFile(string filename,
			 Mesh *mesh,
			 vector<DOFVector<double>*> vecs);

    static void setDofValues(int macroElIndex, Mesh *mesh,
			     vector<double>& values, DOFVector<double>* vec);

    static void readBlock(vector<char> &data,
			  Mesh *mesh,
			  vector<DOFVector<double>*> vecs);

  };
}

#endif
