// ============================================================================
// ==                                                                        ==
// == Amdis - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file ArhWriter.h */

#ifndef AMDIS_ARH_WRITER_H
#define AMDIS_ARH_WRITER_H

#include <iostream>
#include <fstream>

#include "AMDiS_fwd.h"
#include "DOFVector.h"
#include "SystemVector.h"

namespace AMDiS {

  class ArhWriter
  {
  public:
    static void write(std::string filename, Mesh *mesh, 
		      DOFVector<double>* vec0 = NULL, 
		      DOFVector<double>* vec1 = NULL,
		      DOFVector<double>* vec2 = NULL);

    static void write(std::string filename, Mesh *mesh, 
		      std::vector<DOFVector<double>*> vecs,
		      bool writeParallel = true);
    
    static void writeFile(DOFVector<double>* vec0,
			  std::string filename)
    {
	write(filename, vec0->getFeSpace()->getMesh(), vec0);
    }
    
    static void writeFile(DOFVector<double>& vec0,
			  std::string filename)
    {
	write(filename, vec0.getFeSpace()->getMesh(), &vec0);
    }

    static void writeFile(SystemVector *vec,
			  std::string filename)
    {
      std::vector<DOFVector<double>*> vecs;
      for (int i = 0; i < vec->getSize(); i++)
	vecs.push_back(vec->getDOFVector(i));
      write(filename, vecs[0]->getFeSpace()->getMesh(), vecs);
    }

    static void writeFile(std::vector<DOFVector<double>*> vecs,
			  std::string filename)
    {
      write(filename, vecs[0]->getFeSpace()->getMesh(), vecs);
    }
    
  protected:
    static void writeMacroElement(std::ofstream &file, 
				  MeshStructure &code, 
				  std::vector<std::vector<double> >& values,
				  uint32_t elIndex);
  };
}

#endif
