// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file CheckerPartitioner.h */

#ifndef AMDIS_CHECKER_PARTITIONER_H
#define AMDIS_CHECKER_PARTITIONER_H

#include "AMDiS_fwd.h"
#include "Global.h"
#include "Initfile.h"
#include "parallel/MeshPartitioner.h"

namespace AMDiS {

  using namespace std;

  class CheckerPartitioner : public MeshPartitioner
  {
  public:
    CheckerPartitioner(string name, MPI::Intracomm *comm);

    ~CheckerPartitioner() {}

    bool createInitialPartitioning();

    void createTetrahedronStripes();

    /// \ref MeshPartitioner::partition
    bool partition(map<int, double> &elemWeights, PartitionMode mode = INITIAL)
    {
      return true;
    }

    void createPartitionMap(map<int, int>& pMap)
    {
      pMap = partitionMap;
    }

  protected:
    int mpiRank, mpiSize;

    /// 0: standard mode: each node gets one box
    /// 1: x-stripes: each node gets one x-stripe of boxes
    /// 2: y-stripes: each node gets one y-stripe of boxes
    /// 3: z-stripes: each node gets one y-stripe of boxes
    /// 4: tetrahedron-stripes: alias Hieram mode :)
    int mode;

    /// Only used in mode 4.
    map<int, int> elStripeInRank;

    bool multilevel;
  };
}

#endif
